/************************************************************************
 *
 *   IRC - Internet Relay Chat, include/channel.h
 *   Copyright (C) 1990 Jarkko Oikarinen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: channel.h,v 1.7 2001/09/16 22:45:05 asuffield Exp $
 */

#ifndef INCLUDED_channel_h
#define INCLUDED_channel_h
#ifndef INCLUDED_config_h
#include "config.h"		/* config settings */
#endif
#ifndef INCLUDED_ircd_defs_h
#include "ircd_defs.h"		/* buffer sizes */
#endif
#ifndef INCLUDED_sys_types_h
#include <sys/types.h>		/* time_t */
#define INCLUDED_sys_types_h
#endif
#include "umodes.h"
#include "cmodes.h"

struct SLink;
struct Client;

/* This holds the temporary invites
 *  -- asuffield
 */
struct TInvite
{
  struct Client *cptr;
  time_t until;
};

/* channel structure */

struct Channel
{
  struct Channel *nextch;
  struct Channel *prevch;
  struct Channel *hnextch;
  /* Here's the new channel mode stuff... */
  u_int32_t mode_flags;
  int32_t mode_intval[CMODE_INTVAL_COUNT];
  char *mode_strval[CMODE_STRVAL_COUNT];
  struct SLink *mode_mask_list[CMODE_MASK_COUNT];
  char topic[TOPICLEN + 1];
#ifdef TOPIC_INFO
  char topic_nick[NICKLEN + 1];
  time_t topic_time;
#endif
  int users;
  struct SLink *members;
  struct SLink *invites;
  struct SLink *tmemberlist;	/* Members who are only temporarily invited */
  time_t channelts;
  time_t remotets;		/* timestamp on the other side of the burst, -1 if non-existant */
#ifdef FLUD
  time_t fludblock;
  struct fludbot *fluders;
  char lastmsg[32];
  int repeatcount, lastlen;
#endif
  struct Client *frozen_by;	/* This is normally NULL, but is the cptr of the server that
				 * invoked channel freezing on me during a burst */
  char chname[1];
};

extern struct Channel *channel;

#define CREATE 1		/* whether a channel should be
				   created or just tested for existance */

#define MODEBUFLEN      1024	/* Memory is cheap. Let's be safe -- asuffield */

#define NullChn ((struct Channel *)0)

#define ChannelExists(n)        (hash_find_channel(n, NullChn) != NullChn)

extern struct SLink *find_channel_link(struct SLink *, struct Channel *);
extern void remove_user_from_channel(struct Client *, struct Channel *, int);
extern void del_invite(struct Client *, struct Channel *);
extern int can_send(struct Client *, struct Channel *);
extern int is_chan_op(struct Client *, struct Channel *);
extern int has_voice(struct Client *, struct Channel *);
extern int user_channel_mode(struct Client *, struct Channel *);
extern int count_channels(struct Client *);
extern void send_channel_modes(struct Client *, struct Channel *);
extern void del_invite(struct Client *, struct Channel *);
extern int check_channel_name(const char *name);
extern char *pretty_mask(char *);

extern time_t next_temp_invite_due;
extern struct Channel *get_channel(struct Client *, char *, int);
extern void add_user_to_channel(struct Channel *, struct Client *, int);
extern int can_join(struct Client *, struct Channel *, char *, int *);
extern struct Channel *get_ban_channel_forward(struct Client *,
					       struct Channel *);
extern void add_invite(struct Client *, struct Channel *, time_t);
extern int is_invex(struct Client *, struct Channel *);
extern void sub1_from_channel(struct Channel *);

/*
** Channel Related macros follow
*/

#ifdef NEED_SPLITCODE

#if 0
extern int server_was_split;
extern time_t server_split_time;

#ifdef SPLIT_PONG
extern int got_server_pong;
#endif /* SPLIT_PONG */
#endif
#endif /* NEED_SPLITCODE */

#define HoldChannel(x)          (!(x))
/* name invisible */
#define SecretChannel(x)        ((x) && ((x)->mode_flags & CMODE_SECRET))
/* channel not shown but names are */
#define HiddenChannel(x)        ((x) && ((x)->mode_flags & CMODE_SECRET))
/* channel visible */
#define ShowChannel(v,c)        ((c)->users && \
				 (HasUmode(v,UMODE_AUSPEX) || PubChannel(c) || IsMember((v),(c))))
#define PubChannel(x)           ((!x) || ((x)->mode_flags & CMODE_SECRET) == 0)

#define IsMember(blah,chan) ((blah && blah->user && \
                find_channel_link((blah->user)->channel, chan)) ? 1 : 0)

#define IsChannelName(name) ((name) && (*(name) == '#'))

#if defined(INVITE_CHANNEL_FORWARDING) || defined(BAN_CHANNEL_FORWARDING)
#define INVITE_FORWARD 1
#define BAN_FORWARD 2
#endif

#endif /* INCLUDED_channel_h */
